/*	$NetBSD$	*/

/*-
 * Copyright (c) 2013 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * This code is derived from software contributed to The NetBSD Foundation
 * by Steffen Nurpmeso.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

ssize_t
__TXT_FUN(char const *s1, char const *s2 __TXT_NARG __TXT_LOC_DECL)
{
	_TXT_TXTT_DECL(t1)
	_TXT_TXTT_DECL(t2)
	_TXT_BOUNDT_DECL(mbs1)
	_TXT_BOUNDT_DECL(mbs2)
	size_t s1len, s2len;
	ssize_t ret, s1len_save, s2len_save;

	_DIAGASSERT(s1 != NULL);
	_DIAGASSERT(s2 != NULL);

#ifdef __TXT_N_N
	if ((ret = n) == 0)
		goto jleave;
	for (s1len = 0; *s1 != '\0' && s1len < n; ++s1len, ++s1)
		;
	s1 -= s1len;
	s1len_save = s1len;
	for (s2len = 0; *s2 != '\0' && s2len < n; ++s2len, ++s2)
		;
	s2 -= s2len;
	s2len_save = -s2len;
#else
	s1len_save = s1len = strlen(s1);
	s2len_save = -(s2len = strlen(s2));
#endif

	if (s1len == 0) {
		ret = s2len;
		ret = -ret;
		goto jleave;
	} else if (s2len == 0) {
		ret = s1len;
		goto jleave;
	}

	_TXT_TXTT_CREAT(t1);
	_TXT_TXTT_CREAT(t2);
	_TXT_BOUNDT_CREAT(mbs1);
	_TXT_BOUNDT_CREAT(mbs2);
	for (;;) {
		ssize_t cl1, cl2;
		cl1 = __TXT_PEEKSBOUND(t1, s1, s1len, mbs1);
		cl2 = __TXT_PEEKSBOUND(t2, s2, s2len, mbs2);
		if (cl1 == 0) {
			ret = (cl2 == 0) ? 0 :
				(cl2 < 0) ? s1len_save : s2len_save;
			break;
		} else if (cl2 == 0) {
			ret = (cl1 < 0) ? s2len_save : s1len_save;
			break;
		} else if (cl1 < 0) {
			ret = (cl2 < 0) ? 0 : s2len_save;
			break;
		} else if (cl2 < 0) {
			ret = s1len_save;
			break;
		}
		ret = __TXT_TXTT_CMP(t1, t2);
		if (ret > 0) {
			ret = s1len_save;
			break;
		} else if (ret < 0) {
			ret = s2len_save;
			break;
		}
		s1 += cl1;
		s2 += cl2;
		s1len -= cl1;	/* XXX should we avoid another peek .. */
		s2len -= cl2;	/* XXX if any runs 0 here? */
	}
	_TXT_BOUNDT_GUT(mbs2);
	_TXT_BOUNDT_GUT(mbs1);
	_TXT_TXTT_GUT(t2);
	_TXT_TXTT_GUT(t1);
jleave:
	return ret;
}
