/*	$NetBSD$	*/

/*-
 * Copyright (c) 2013 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * This code is derived from software contributed to The NetBSD Foundation
 * by Steffen Nurpmeso.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

ssize_t
__TXT_FUN(char *restrict dst, size_t size, char const *restrict src,
	size_t srclen, size_t *dstlen_final __TXT_LOC_DECL)
{
	_TXT_TXTT_DECL(t)
	_TXT_BOUNDT_DECL(mbs)
	ssize_t targlen;
	size_t dstlen;

	_DIAGASSERT(dst != NULL);
	_DIAGASSERT(src != NULL);

	dstlen = 0;
	targlen = srclen;

	if (size == 0)
		goto jleave;
	if (srclen == 0)
		goto jnul;

	_TXT_TXTT_CREAT(t);
	_TXT_BOUNDT_CREAT(mbs);
	for (;;) {
		ssize_t cl = __TXT_PEEKSBOUND(t, src + dstlen, srclen, mbs);
		if (cl == 0) /* XXX ? */
			break;
		if (cl < 0) {
			targlen = -(dstlen + 1);
			break;
		}
		if ((dstlen += cl) >= size) {
			dstlen -= cl;
			break;
		}
		if ((srclen -= cl) == 0)
			break;
	}
	_TXT_BOUNDT_GUT(mbs);
	_TXT_TXTT_GUT(t);

	memcpy(dst, src, dstlen);
jnul:
	dst[dstlen] = '\0';
jleave:
	if (dstlen_final != NULL)
		*dstlen_final = dstlen;
	return targlen;
}
