/*	$NetBSD$	*/

/*-
 * Copyright (c) 2013 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * This code is derived from software contributed to The NetBSD Foundation
 * by Steffen Nurpmeso.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

ssize_t
__TXT_FUN(char *restrict dst, char const *restrict src, size_t n __TXT_LOC_DECL)
{
	_TXT_TXTT_DECL(t)
	_TXT_BOUNDT_DECL(mbs)
	char err = 0;
	size_t len = 0, srclen;

	_DIAGASSERT(dst != NULL);
	_DIAGASSERT(src != NULL);

	if (n == 0)
		goto jleave;

	if ((srclen = strlen(src)) == 0)
		goto jnul;

	_TXT_TXTT_CREAT(t);
	_TXT_BOUNDT_CREAT(mbs);
	for (;;) {
		ssize_t cl = __TXT_PEEKSBOUND(t, src + len, srclen, mbs);
		if (cl == 0) /* XXX ? */
			break;
		if (cl < 0) {
			err = 1;
			break;
		}
		if ((len += cl) >= n) {
			if (len > n)
				len -= cl;
			break;
		}
		if ((srclen -= cl) == 0)
			break;
	}
	_TXT_BOUNDT_GUT(mbs);
	_TXT_TXTT_GUT(t);

	memcpy(dst, src, len);
	dst += len;
	n -= len;
jnul:
	if (n > 0)
		memset(dst, '\0', n);
jleave:
	return err ? -(len + 1) : len;
}
