/*	$NetBSD$	*/

/*-
 * Copyright (c) 2013 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * This code is derived from software contributed to The NetBSD Foundation
 * by Steffen Nurpmeso.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

char const *
__TXT_FUN(char const *s1 __TXT_LARG(s1len), char const *s2 __TXT_LARG(s2len)
	__TXT_LOC_DECL)
{
#if __TXT_CASE != 0
	_TXT_TXTT_DECL(t2c1)
#endif
	_TXT_TXTT_DECL(t)
	_TXT_BOUNDT_DECL(mbs)
	char const *ret;
#ifndef __TXT_L_L
	size_t s1len, s2len;
#endif
	ssize_t i;

	_DIAGASSERT(s1 != NULL);
	_DIAGASSERT(s2 != NULL);

#if __TXT_CASE != 0
	_TXT_TXTT_CREAT_PSSST_CC(t2c1);
#endif
#ifndef __TXT_L_L
	s1len = strlen(s1);
	s2len = strlen(s2);
#endif
	ret = s1;
	if (s2len == 0)
		goto jleave;
	ret = NULL;
	if (s1len < s2len)
		goto jleave;

	/*
	 * Check s2 for validity
	 */
	_TXT_TXTT_CREAT(t);
	_TXT_BOUNDT_CREAT(mbs);
	for (ret = s2, i = s2len; i > 0;) {
		ssize_t cl = __TXT_PEEKSBOUND(t, ret, i, mbs);
		if (cl == 0)
			break;
		if (cl < 0) {
			i = -1;
			break;
		}
#if __TXT_CASE != 0
		if (ret == s2)
			_TXT_TXTT_COPY(t2c1, t);
#endif
		i -= cl;
		ret += cl;
	}
	ret = NULL;
	_TXT_BOUNDT_GUT(mbs);
	_TXT_TXTT_GUT(t);

	if (i != 0) /* XXX really bail if s2len > strlen(s2)? */
		goto jleave;

	_TXT_TXTT_CREAT(t);
	_TXT_BOUNDT_CREAT(mbs);
	while (s1len >= s2len) {
		i = __TXT_PEEKSBOUND(t, s1, s1len, mbs);
		if (i <= 0) /* XXX ? */
			break;
		/*
		 * We know that s2 is valid.
		 * If we're not doing the case-insensitive search, a simple
		 * by-byte comparison will do the job once we have found
		 * a matching starting byte.
		 * Otherwise, for TEXT_SUPPORT_LEVEL<3, go the simple route and
		 * call txtncasecmp(_l)?()
		 */
		if (
#if __TXT_CASE == 0
			*s1 == *s2 && memcmp(s1, s2, s2len) == 0
#else
			__TXT_TXTT_CMP(t, t2c1) == 0 &&
# if __TXT_CASE == 1
				txtncasecmp(s1, s2, s2len) == 0
# else
				txtncasecmp_l(s1, s2, s2len, loc) == 0
# endif
#endif
		) {
			ret = s1;
#ifndef __TXT_REVERSE
			break;
#endif
		}
		s1 += i;
		s1len -= i;
	}
	_TXT_BOUNDT_GUT(mbs);
	_TXT_TXTT_GUT(t);
jleave:
#if __TXT_CASE != 0
	_TXT_TXTT_GUT(t2c1);
#endif
	return ret;
}
