/*	$NetBSD$	*/

/*-
 * Copyright (c) 2013 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * This code is derived from software contributed to The NetBSD Foundation
 * by Steffen Nurpmeso.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <assert.h>
#include <stdlib.h>
#include <string.h>

#if TEXT_SUPPORT_LEVEL == 1
# include <wchar.h>
# include <wctype.h>
#else
# error Unsupported TEXT_SUPPORT_LEVEL
#endif

#ifndef _DIAGASSERT
# define _DIAGASSERT(X)			assert(X)
#endif

#if TEXT_SUPPORT_LEVEL == 1
# define _TXT_TXTT			wchar_t
# define _TXT_TXTT_DECL(N)		wchar_t N;
# define _TXT_TXTT_CREAT(N)
# define _TXT_TXTT_CREAT_PSSST_CC(N)	do { N = 0; } while (0)
# define _TXT_TXTT_COPY(DST,SRC)	do { DST = SRC; } while (0)
# define _TXT_TXTT_GUT(N)
# define _TXT_TXTT_ARG(N)		, &(N)
# define _TXT_TXTT_CMP(T1,T2)		((ssize_t)(T1) - (ssize_t)(T2))
# define _TXT_TXTT_CASECMP(T1,T2)	\
	((ssize_t)(towlower(T1)) - (ssize_t)(towlower(T2)))
# define _TXT_TXTT_RUNEOF(N)		(N)

# define _TXT_BOUNDT			mbstate_t
# define _TXT_BOUNDT_DECL(N)		mbstate_t N;
# define _TXT_BOUNDT_CREAT(N)		memset(&(N), 0, sizeof(N))
# define _TXT_BOUNDT_RESET		_TXT_BOUNDT_CREAT
# define _TXT_BOUNDT_GUT(N)
# define _TXT_BOUNDT_ARG(N)		, &(N)

# define _TXT_PEEKSBOUND(T,S,L,B)	_txt_peeksbound(&(T), S, L, &(B))

# define _TXT_ISTXY(NAME,T,S,L)		isw ## NAME(T)
# define _TXT_TOTXY(NAME,T,D,S,L)	_txt_totxy(tow ## NAME(T), T, D, S, L)

# ifdef TEXT_SUPPORT_XLOCALE
#  define _TXT_LOC_DECL			locale_t loc

#  define _TXT_TXTT_CASECMP_L(T1,T2)	\
	((ssize_t)(towlower_l(T1, loc)) - (ssize_t)(towlower_l(T2, loc)))

#  define _TXT_PEEKSBOUND_L(T,S,L,B)	_txt_peeksbound_l(&(T), S, L, &(B), loc)

#  define _TXT_ISTXY_L(NAME,T,S,L)	isw ## NAME ## _l(T, loc)
#  define _TXT_TOTXY_L(NAME,T,D,S,L)	\
	_txt_totxy_l(tow ## NAME ## _l(T, loc), T, D, S, L, loc)
# endif

static inline ssize_t
_txt_peeksbound(_TXT_TXTT *wc, char const *s, size_t l, _TXT_BOUNDT *mbs)
{
	return (ssize_t)mbrtowc(wc, s, l, mbs);
}

# ifdef TEXT_SUPPORT_XLOCALE
static inline ssize_t
_txt_peeksbound_l(_TXT_TXTT *wc, char const *s, size_t l, _TXT_BOUNDT *mbs,
	locale_t loc)
{
	return (ssize_t)mbrtowc_l(wc, s, l, mbs, loc);
}
# endif

static inline ssize_t
_txt_totxy(_TXT_TXTT cnv, _TXT_TXTT orig, char *dst, char const *src,
	size_t len)
{
	ssize_t ret;
	if (cnv == orig) {
		memcpy(dst, src, len);
		ret = len;
	} else
		ret = wctomb(dst, cnv);
	return ret;
}

# ifdef TEXT_SUPPORT_XLOCALE
static inline ssize_t
_txt_totxy_l(_TXT_TXTT cnv, _TXT_TXTT orig, char *dst, char const *src,
	size_t len, locale_t loc)
{
	ssize_t ret;
	if (cnv == orig) {
		memcpy(dst, src, len);
		ret = len;
	} else
		ret = wctomb_l(dst, cnv, loc);
	return ret;
}
# endif
#endif /* TEXT_SUPPORT_LEVEL==1 */
