/*	$NetBSD$	*/

/*-
 * Copyright (c) 2013 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * This code is derived from software contributed to The NetBSD Foundation
 * by Steffen Nurpmeso.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <text.h>

#if defined LIBC_SCCS && ! defined lint
__RCSID("$NetBSD$");
#endif

#include "local.h"

/*
 * txtn?bound(_l)?() series.
 * Query the length of a multibyte/multi-codepoint character in bytes.
 * (No more than n bytes are inspected.)
 * Return a negative value in case an illegal or incomplete sequence is seen.
 * 0 is returned if the string is the empty string (or n is 0).
 * Otherwise the length of the next multibyte/multi-codepoint character is
 * returned in bytes:
 */

#if TEXT_SUPPORT_LEVEL > 2
# error Please write an UTF-8 optimized version of the ncpy()s for LEVEL>2

#else /* TEXT_SUPPORT_LEVEL>2 */
# define __TXT_FUN		txtbound
# define __TXT_NARG
# define __TXT_LOC_DECL
# define __TXT_PEEKSBOUND	_TXT_PEEKSBOUND
# include "_bound.h"

# ifdef TEXT_SUPPORT_XLOCALE
#  undef __TXT_FUN
#  define __TXT_FUN		txtbound_l
#  undef __TXT_NARG
#  define __TXT_NARG
#  undef __TXT_N_N
#  undef __TXT_LOC_DECL
#  define __TXT_LOC_DECL	, _TXT_LOC_DECL
#  undef __TXT_PEEKSBOUND
#  define __TXT_PEEKSBOUND	_TXT_PEEKSBOUND_L
#  include "_bound.h"
# endif

# undef __TXT_FUN
# define __TXT_FUN		txtnbound
# undef __TXT_NARG
# define __TXT_NARG		, size_t n
# undef __TXT_N_N
# define __TXT_N_N
# undef __TXT_LOC_DECL
# define __TXT_LOC_DECL
# undef __TXT_PEEKSBOUND
# define __TXT_PEEKSBOUND	_TXT_PEEKSBOUND
# include "_bound.h"

# ifdef TEXT_SUPPORT_XLOCALE
#  undef __TXT_FUN
#  define __TXT_FUN		txtnbound_l
#  undef __TXT_NARG
#  define __TXT_NARG		, size_t n
#  undef __TXT_N_N
#  define __TXT_N_N
#  undef __TXT_LOC_DECL
#  define __TXT_LOC_DECL	, _TXT_LOC_DECL
#  undef __TXT_PEEKSBOUND
#  define __TXT_PEEKSBOUND	_TXT_PEEKSBOUND_L
#  include "_bound.h"
# endif
#endif /* TEXT_SUPPORT_LEVEL>2 */
