/*	$NetBSD$	*/

/*-
 * Copyright (c) 2013 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * This code is derived from software contributed to The NetBSD Foundation
 * by Steffen Nurpmeso.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <text.h>

#if defined LIBC_SCCS && ! defined lint
__RCSID("$NetBSD$");
#endif

#include "local.h"

/*
 * txtn?(case)?cmp(_l)?()
 * Compare (case-insensitive) (no more than n bytes of) the two strings s1 and
 * s2.
 * Only complete characters will be compared; an invalid/incomplete character
 * is treated as if the end of the string had been reached;
 * see txtl(case)?cmp() for a solution that reports such errors. TODO not yet
 * For txtn(case)?cmp(), n is treated as a hard cut for the string lengths.
 * Returns 0 if both strings are compare equal;
 * otherwise it returns the length of the "greater" string, returning
 * a negative value if s1 compares "less" than s2.
 */

#if TEXT_SUPPORT_LEVEL > 2
# error Please write an UTF-8 optimized version of the comparisons for LEVEL>2

#else /* TEXT_SUPPORT_LEVEL>2 */
# define __TXT_FUN		txtcmp
# define __TXT_NARG
# define __TXT_LOC_DECL
# define __TXT_TXTT_CMP		_TXT_TXTT_CMP
# define __TXT_PEEKSBOUND	_TXT_PEEKSBOUND
# include "_cmp.h"

# ifdef TEXT_SUPPORT_XLOCALE
#  undef __TXT_FUN
#  define __TXT_FUN		txtcmp_l
#  undef __TXT_NARG
#  define __TXT_NARG
#  undef __TXT_N_N
#  undef __TXT_LOC_DECL
#  define __TXT_LOC_DECL	, _TXT_LOC_DECL
#  undef __TXT_TXTT_CMP
#  define __TXT_TXTT_CMP	_TXT_TXTT_CMP
#  undef __TXT_PEEKSBOUND
#  define __TXT_PEEKSBOUND	_TXT_PEEKSBOUND_L
#  include "_cmp.h"
# endif

# undef __TXT_FUN
# define __TXT_FUN		txtncmp
# undef __TXT_NARG
# define __TXT_NARG		, size_t n
# undef __TXT_N_N
# define __TXT_N_N
# undef __TXT_LOC_DECL
# define __TXT_LOC_DECL
# undef __TXT_TXTT_CMP
# define __TXT_TXTT_CMP		_TXT_TXTT_CMP
# undef __TXT_PEEKSBOUND
# define __TXT_PEEKSBOUND	_TXT_PEEKSBOUND
# include "_cmp.h"

# ifdef TEXT_SUPPORT_XLOCALE
#  undef __TXT_FUN
#  define __TXT_FUN		txtncmp_l
#  undef __TXT_NARG
#  define __TXT_NARG		, size_t n
#  undef __TXT_N_N
#  define __TXT_N_N
#  undef __TXT_LOC_DECL
#  define __TXT_LOC_DECL	, _TXT_LOC_DECL
#  undef __TXT_TXTT_CMP
#  define __TXT_TXTT_CMP	_TXT_TXTT_CMP
#  undef __TXT_PEEKSBOUND
#  define __TXT_PEEKSBOUND	_TXT_PEEKSBOUND_L
#  include "_cmp.h"
# endif
 
# undef __TXT_FUN
# define __TXT_FUN		txtcasecmp
# undef __TXT_NARG
# define __TXT_NARG
# undef __TXT_N_N
# undef __TXT_LOC_DECL
# define __TXT_LOC_DECL
# undef __TXT_TXTT_CMP
# define __TXT_TXTT_CMP		_TXT_TXTT_CASECMP
# undef __TXT_PEEKSBOUND
# define __TXT_PEEKSBOUND	_TXT_PEEKSBOUND
# include "_cmp.h"

# ifdef TEXT_SUPPORT_XLOCALE
#  undef __TXT_FUN
#  define __TXT_FUN		txtcasecmp_l
#  undef __TXT_NARG
#  define __TXT_NARG
#  undef __TXT_N_N
#  undef __TXT_LOC_DECL
#  define __TXT_LOC_DECL	, _TXT_LOC_DECL
#  undef __TXT_TXTT_CMP
#  define __TXT_TXTT_CMP	_TXT_TXTT_CASECMP_L
#  undef __TXT_PEEKSBOUND
#  define __TXT_PEEKSBOUND	_TXT_PEEKSBOUND_L
#  include "_cmp.h"
# endif

# undef __TXT_FUN
# define __TXT_FUN		txtncasecmp
# undef __TXT_NARG
# define __TXT_NARG		, size_t n
# undef __TXT_N_N
# define __TXT_N_N
# undef __TXT_LOC_DECL
# define __TXT_LOC_DECL
# undef __TXT_TXTT_CMP
# define __TXT_TXTT_CMP		_TXT_TXTT_CASECMP
# undef __TXT_PEEKSBOUND
# define __TXT_PEEKSBOUND	_TXT_PEEKSBOUND
# include "_cmp.h"

# ifdef TEXT_SUPPORT_XLOCALE
#  undef __TXT_FUN
#  define __TXT_FUN		txtncasecmp_l
#  undef __TXT_NARG
#  define __TXT_NARG		, size_t n
#  undef __TXT_N_N
#  define __TXT_N_N
#  undef __TXT_LOC_DECL
#  define __TXT_LOC_DECL	, _TXT_LOC_DECL
#  undef __TXT_TXTT_CMP
#  define __TXT_TXTT_CMP	_TXT_TXTT_CASECMP_L
#  undef __TXT_PEEKSBOUND
#  define __TXT_PEEKSBOUND	_TXT_PEEKSBOUND_L
#  include "_cmp.h"
# endif
#endif /* TEXT_SUPPORT_LEVEL>2 */
