/*	$NetBSD$	*/

/*-
 * Copyright (c) 2013 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * This code is derived from software contributed to The NetBSD Foundation
 * by Steffen Nurpmeso.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <text.h>

#if defined LIBC_SCCS && ! defined lint
__RCSID("$NetBSD$");
#endif

#include "local.h"

/*
 * Duplicate src.
 * Does not check multibyte sequences for validity.
 * Returns a terminated duplicate, or NULL (with ENOMEM set) on allocation
 * failure.
 */
char *
txtdup(char const *src)
{
	char *dst;
	size_t l;

	_DIAGASSERT(src != NULL);

	l = strlen(src) + 1;
	if ((dst = malloc(l)) != NULL)
		memcpy(dst, src, l);
	/* else: POSIX requires malloc(3) to set ENOMEM */

	return dst;
}

/*
 * txtndup(_l)?() series.
 * Duplicate no more than n bytes of src.
 * Only complete characters will be stored in dst;
 * Returns the terminated duplicate, or NULL (with ENOMEM set) on allocation
 * failure.
 * If the len argument is not NULL, it'll be set to the length of dst.
 * Except for length comparison there is no error indicator.
 */

#define __TXT_FUN		txtndup
#define __TXT_LOC_DECL
#define __TXT_PEEKSBOUND	_TXT_PEEKSBOUND
#include "_dup.h"

#ifdef TEXT_SUPPORT_XLOCALE
# undef __TXT_FUN
# define __TXT_FUN		txtndup_l
# undef __TXT_LOC_DECL
# define __TXT_LOC_DECL		, _TXT_LOC_DECL
# undef __TXT_PEEKSBOUND
# define __TXT_PEEKSBOUND	_TXT_PEEKSBOUND_L
# include "_dup.h"
#endif
