/*	$NetBSD$	*/

/*-
 * Copyright (c) 2013 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * This code is derived from software contributed to The NetBSD Foundation
 * by Steffen Nurpmeso.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <text.h>

#if defined LIBC_SCCS && ! defined lint
__RCSID("$NetBSD$");
#endif

#include "local.h"

/*
 * txtlcpy(_l)?() series.
 * Copy no more than size-1 bytes of src into dst.
 * dst will always be terminated unless size was 0.
 * Copying stops once a terminating NUL is encountered in src;
 * *note* that it is required that src is NUL terminated.
 * Only complete characters will be stored in dst.
 * Returns the length of src; truncation occurred if this is >=size;
 * if the return value is negative then copying stopped because there was an
 * illegal/incomplete multibyte character in src, in which case the return
 * value denotes the failing byte, counting from 1, in src.
 * If dstlen_final is not NULL it is set to the resulting length of dst.
 */

#if TEXT_SUPPORT_LEVEL > 2
# error Please write an UTF-8 optimized version of the lcpy()s for LEVEL>2

#else /* TEXT_SUPPORT_LEVEL>2 */
# define __TXT_FUN		txtlcpy
# define __TXT_LOC_DECL
# define __TXT_PEEKSBOUND	_TXT_PEEKSBOUND
# include "_lcpy.h"

# ifdef TEXT_SUPPORT_XLOCALE
#  undef __TXT_FUN
#  define __TXT_FUN		txtlcpy_l
#  undef __TXT_LOC_DECL
#  define __TXT_LOC_DECL	, _TXT_LOC_DECL
#  undef __TXT_PEEKSBOUND
#  define __TXT_PEEKSBOUND	_TXT_PEEKSBOUND_L
#  include "_lcpy.h"
# endif
#endif /* TEXT_SUPPORT_LEVEL>2 */
