/*	$NetBSD$	*/

/*-
 * Copyright (c) 2013 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * This code is derived from software contributed to The NetBSD Foundation
 * by Steffen Nurpmeso.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <text.h>

#if defined LIBC_SCCS && ! defined lint
__RCSID("$NetBSD$");
#endif

#include "local.h"

/*
 * txtncpy(_l)?() series.
 * Copy no more than n bytes of src into dst.
 * Copying stops once a terminating NUL is encountered in src;
 * *note* that it is required that src is NUL terminated.
 * If the n bytes are not completely consumed after the copying is completed,
 * the remainder of dst is filled with NUL characters.
 * Only complete (multibyte) characters will be stored in dst.
 * Return the new length of dst;
 * if this equals n then no NUL has been encountered and dst is not terminated;
 * if the length is returned as a negative value then copying stopped because
 * there was an illegal/incomplete multibyte character in src -- possible NUL
 * filling takes place regardless of such an error.
 */

#if TEXT_SUPPORT_LEVEL > 2
# error Please write an UTF-8 optimized version of the ncpy()s for LEVEL>2

#else /* TEXT_SUPPORT_LEVEL>2 */
# define __TXT_FUN		txtncpy
# define __TXT_LOC_DECL
# define __TXT_PEEKSBOUND	_TXT_PEEKSBOUND
# include "_ncpy.h"

# ifdef TEXT_SUPPORT_XLOCALE
#  undef __TXT_FUN
#  define __TXT_FUN		txtncpy_l
#  undef __TXT_LOC_DECL
#  define __TXT_LOC_DECL	, _TXT_LOC_DECL
#  undef __TXT_PEEKSBOUND
#  define __TXT_PEEKSBOUND	_TXT_PEEKSBOUND_L
#  include "_ncpy.h"
# endif
#endif /* TEXT_SUPPORT_LEVEL>2 */
