/*	$NetBSD$	*/

/*-
 * Copyright (c) 2013 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * This code is derived from software contributed to The NetBSD Foundation
 * by Steffen Nurpmeso.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <text.h>

#if defined LIBC_SCCS && ! defined lint
__RCSID("$NetBSD$");
#endif

#include "local.h"

/*
 * txt(ll)?r?(case)?str(_l)?()
 * (Reverse) Search (case-insensitively) a substring (of given length 2) in
 * a string (of given length 1).
 * Only complete characters will be compared; an invalid/incomplete character
 * is treated as if the end of the string had been reached.
 * An empty substring always matches, even an empty string 1.
 * A pointer to the start of the (last) matching substring in string is
 * returned on success, but in all other cases, including premature abort due
 * to invalid byte sequences, the NULL pointer is returned.
 */

#if TEXT_SUPPORT_LEVEL > 2
# error Please write an UTF-8 optimized version of the *str*s for LEVEL>2

#else /* TEXT_SUPPORT_LEVEL>2 */
 /*
  * "Forward"
  */
# undef __TXT_REVERSE

# define __TXT_CASE		0

# define __TXT_LOC_DECL
# define __TXT_TXTT_CMP		_TXT_TXTT_CMP
# define __TXT_PEEKSBOUND	_TXT_PEEKSBOUND

# define __TXT_FUN		txtstr
# define __TXT_LARG(N)
# include "_str.h"

# undef __TXT_FUN
# define __TXT_FUN		txtllstr
# undef __TXT_LARG
# define __TXT_LARG(N)		, size_t N
# define __TXT_L_L
# include "_str.h"

# ifdef TEXT_SUPPORT_XLOCALE
#  undef __TXT_LOC_DECL
#  define __TXT_LOC_DECL	, _TXT_LOC_DECL
#  undef __TXT_TXTT_CMP
#  define __TXT_TXTT_CMP	_TXT_TXTT_CMP_L
#  undef __TXT_PEEKSBOUND
#  define __TXT_PEEKSBOUND	_TXT_PEEKSBOUND_L
#  undef __TXT_L_L

#  undef __TXT_FUN
#  define __TXT_FUN		txtstr_l
#  undef __TXT_LARG
#  define __TXT_LARG(N)
#  include "_str.h"

#  undef __TXT_FUN
#  define __TXT_FUN		txtllstr_l
#  undef __TXT_LARG
#  define __TXT_LARG(N)		, size_t N
#  define __TXT_L_L
#  include "_str.h"
# endif

# undef __TXT_CASE
# define __TXT_CASE		1

# undef __TXT_LOC_DECL
# define __TXT_LOC_DECL
# undef __TXT_TXTT_CMP
# define __TXT_TXTT_CMP		_TXT_TXTT_CASECMP
# undef __TXT_PEEKSBOUND
# define __TXT_PEEKSBOUND	_TXT_PEEKSBOUND
# undef __TXT_L_L

# undef __TXT_FUN
# define __TXT_FUN		txtcasestr
# undef __TXT_LARG
# define __TXT_LARG(N)
# include "_str.h"

# undef __TXT_FUN
# define __TXT_FUN		txtllcasestr
# undef __TXT_LARG
# define __TXT_LARG(N)		, size_t N
# define __TXT_L_L
# include "_str.h"

# undef __TXT_CASE
# define __TXT_CASE		2

# ifdef TEXT_SUPPORT_XLOCALE
#  undef __TXT_LOC_DECL
#  define __TXT_LOC_DECL	, _TXT_LOC_DECL
#  undef __TXT_TXTT_CMP
#  define __TXT_TXTT_CMP	_TXT_TXTT_CASECMP_L
#  undef __TXT_PEEKSBOUND
#  define __TXT_PEEKSBOUND	_TXT_PEEKSBOUND_L
#  undef __TXT_L_L

#  undef __TXT_FUN
#  define __TXT_FUN		txtcasestr_l
#  undef __TXT_LARG
#  define __TXT_LARG(N)
#  include "_str.h"

#  undef __TXT_FUN
#  define __TXT_FUN		txtllcasestr_l
#  undef __TXT_LARG
#  define __TXT_LARG(N)		, size_t N
#  define __TXT_L_L
#  include "_str.h"
# endif

 /*
  * "Reverse"
  */
# define __TXT_REVERSE

# undef __TXT_CASE
# define __TXT_CASE		0

# undef __TXT_LOC_DECL
# define __TXT_LOC_DECL
# undef __TXT_TXTT_CMP
# define __TXT_TXTT_CMP		_TXT_TXTT_CMP
# undef __TXT_PEEKSBOUND
# define __TXT_PEEKSBOUND	_TXT_PEEKSBOUND
# undef __TXT_L_L

# undef __TXT_FUN
# define __TXT_FUN		txtrstr
# undef __TXT_LARG
# define __TXT_LARG(N)
# include "_str.h"

# undef __TXT_FUN
# define __TXT_FUN		txtllrstr
# undef __TXT_LARG
# define __TXT_LARG(N)		, size_t N
# define __TXT_L_L
# include "_str.h"

# ifdef TEXT_SUPPORT_XLOCALE
#  undef __TXT_LOC_DECL
#  define __TXT_LOC_DECL	, _TXT_LOC_DECL
#  undef __TXT_TXTT_CMP
#  define __TXT_TXTT_CMP	_TXT_TXTT_CMP_L
#  undef __TXT_PEEKSBOUND
#  define __TXT_PEEKSBOUND	_TXT_PEEKSBOUND_L
#  undef __TXT_L_L

#  undef __TXT_FUN
#  define __TXT_FUN		txtrstr_l
#  undef __TXT_LARG
#  define __TXT_LARG(N)
#  include "_str.h"

#  undef __TXT_FUN
#  define __TXT_FUN		txtllrstr_l
#  undef __TXT_LARG
#  define __TXT_LARG(N)		, size_t N
#  define __TXT_L_L
#  include "_str.h"
# endif

# undef __TXT_CASE
# define __TXT_CASE		1

# undef __TXT_LOC_DECL
# define __TXT_LOC_DECL
# undef __TXT_TXTT_CMP
# define __TXT_TXTT_CMP		_TXT_TXTT_CASECMP
# undef __TXT_PEEKSBOUND
# define __TXT_PEEKSBOUND	_TXT_PEEKSBOUND
# undef __TXT_L_L

# undef __TXT_FUN
# define __TXT_FUN		txtrcasestr
# undef __TXT_LARG
# define __TXT_LARG(N)
# include "_str.h"

# undef __TXT_FUN
# define __TXT_FUN		txtllrcasestr
# undef __TXT_LARG
# define __TXT_LARG(N)		, size_t N
# define __TXT_L_L
# include "_str.h"

# undef __TXT_CASE
# define __TXT_CASE		2

# ifdef TEXT_SUPPORT_XLOCALE
#  undef __TXT_LOC_DECL
#  define __TXT_LOC_DECL	, _TXT_LOC_DECL
#  undef __TXT_TXTT_CMP
#  define __TXT_TXTT_CMP	_TXT_TXTT_CASECMP_L
#  undef __TXT_PEEKSBOUND
#  define __TXT_PEEKSBOUND	_TXT_PEEKSBOUND_L
#  undef __TXT_L_L

#  undef __TXT_FUN
#  define __TXT_FUN		txtrcasestr_l
#  undef __TXT_LARG
#  define __TXT_LARG(N)
#  include "_str.h"

#  undef __TXT_FUN
#  define __TXT_FUN		txtllrcasestr_l
#  undef __TXT_LARG
#  define __TXT_LARG(N)		, size_t N
#  define __TXT_L_L
#  include "_str.h"
# endif
#endif /* TEXT_SUPPORT_LEVEL>2 */
