/*	$NetBSD$	*/

/*-
 * Copyright (c) 2013 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * This code is derived from software contributed to The NetBSD Foundation
 * by Steffen Nurpmeso.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <text.h>

#if defined LIBC_SCCS && ! defined lint
__RCSID("$NetBSD$");
#endif

#include "local.h"

/*
 * Character transformation series.
 * totl?XY(_l)?():
 *	Transform the multibyte/multi-codepoint sequence src and store the
 *	result in dst, which should be at least TXT_BOUND_MAX bytes.
 *	If no transformation applies the src is copied unchanged.
 *	These return a value <0 if the (first n bytes of the) given string
 *	do(es) not form a valid character, 0 if src is empty / n was 0, and the
 *	length of the copied character in bytes otherwise.
 *	src will always be terminated.
 */

#define __TXT_FUN(N)		tot ## N
#define __TXT_LOC_DECL
#define __TXT_PEEKSBOUND	_TXT_PEEKSBOUND
#define __TXT_TOTXY		_TXT_TOTXY
#include "_trans.h"

#ifdef TEXT_SUPPORT_XLOCALE
# undef __TXT_FUN
# define __TXT_FUN(N)		tot ## N ## _l
# undef __TXT_LOC_DECL
# define __TXT_LOC_DECL		, _TXT_LOC_DECL
# undef __TXT_PEEKSBOUND
# define __TXT_PEEKSBOUND	_TXT_PEEKSBOUND_L
# undef __TXT_TOTXY
# define __TXT_TOTXY		_TXT_TOTXY_L
# include "_trans.h"
#endif
