/*
 * Written by Steffen Nurpmeso.
 * Public domain.
 */

#include <locale.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "text.h"

struct tdat {
	int no;
	char const *s;
	size_t n;
	ssize_t ret;
};

static struct tdat const	_td1_posix[] = {
	{ 0, "", 0, 0 },
	{ 1, "a", 1, 1 },
	{ 2, "A", 1, 1 },
	{ 3, "\t", 1, 1 },
#ifdef TEXT_SUPPORT_8BIT_C_LOCALE
	{ 4, "\303\244", 2, 1 },
	{ 5, "\370\244", 2, 1 },
	{ 6, "\341\240\200", 3, 1 },
	{ 7, "\361\240\200", 3, 1 },
#endif
	{ 0, NULL, 0, 0 }
},
			_td1_utf8[] = {
	{ 0, "", 0, 0 },
	{ 1, "a", 1, 1 },
	{ 2, "A", 1, 1 },
	{ 3, "\t", 1, 1 },
	{ 4, "\303\244", 2, 2 },
	{ 5, "\370\244", 2, -2 },
	{ 6, "\341\240\200", 3, 3 },
	{ 7, "\361\240\200", 3, -2 },
	{ 0, NULL, 0, 0 }
};

static char const *	_ts;
static ssize_t	(*	_tptf)(char const *, size_t);
#ifdef TEXT_SUPPORT_XLOCALE
static locale_t		_lc;
#endif

static void	_t1(int tno, struct tdat const *);

static void
_t1(int tno, struct tdat const *td)
{
	struct tdat const *x;
	ssize_t r;

	for (x = td; x->s != NULL; ++x) {
		r = (*_tptf)(x->s, x->n);
		if ((r >= 0 && r != x->ret) || (r < 0 && x->ret >= 0))
			atf_tc_fail("%d.%d %s(): got %zd, should be %zd\n",
				tno, x->no, _ts, r, x->ret);
	}
}

static ssize_t
_tgo(char const *s, size_t n)
{ (void)n; return txtbound(s); }

ATF_TC_BODY(txtbound_go, tc)
{
	_tptf = &_tgo;

	_ts = "POSIX:txtbound";
	if (setlocale(LC_CTYPE, "POSIX") == 0)
		atf_tc_fail("1. %s: cannot set locale\n", _ts);
	_t1(1, _td1_posix);

	_ts = "en_USSR.UTF-8:txtbound";
	if (setlocale(LC_CTYPE, "en_US.UTF-8") == 0)
		atf_tc_fail("1. %s: cannot set locale\n", _ts);
	_t1(2, _td1_utf8);
}

#ifdef TEXT_SUPPORT_XLOCALE
static ssize_t
_tgo_l(char const *s, size_t n)
{ (void)n; return txtbound_l(s, _lc); }

ATF_TC_BODY(txtbound_l_go, tc)
{
	_tptf = &_tgo_l;

	_ts = "POSIX:txtbound_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "POSIX", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("2. %s: cannot newlocale()\n", _ts);
	_t1(3, _td1_posix);

	_ts = "en_USSR.UTF-8:txtbound_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "en_US.UTF-8", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("2. %s: cannot newlocale()\n", _ts);
	_t1(4, _td1_utf8);
}
#endif /* TEXT_SUPPORT_XLOCALE */

static ssize_t
_tngo(char const *s, size_t n)
{ return txtnbound(s, n); }

ATF_TC_BODY(txtnbound_go, tc)
{
	_tptf = &_tngo;

	_ts = "POSIX:txtnbound";
	if (setlocale(LC_CTYPE, "POSIX") == 0)
		atf_tc_fail("3. %s: cannot set locale\n", _ts);
	_t1(5, _td1_posix);

	_ts = "en_USSR.UTF-8:txtnbound";
	if (setlocale(LC_CTYPE, "en_US.UTF-8") == 0)
		atf_tc_fail("3. %s: cannot set locale\n", _ts);
	_t1(6, _td1_utf8);
}

#ifdef TEXT_SUPPORT_XLOCALE
static ssize_t
_tngo_l(char const *s, size_t n)
{ return txtnbound_l(s, n, _lc); }

ATF_TC_BODY(txtnbound_l_go, tc)
{
	_tptf = &_tngo_l;

	_ts = "POSIX:txtnbound_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "POSIX", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("4. %s: cannot newlocale()\n", _ts);
	_t1(7, _td1_posix);

	_ts = "en_USSR.UTF-8:txtnbound_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "en_US.UTF-8", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("4. %s: cannot newlocale()\n", _ts);
	_t1(8, _td1_utf8);
}
#endif /* TEXT_SUPPORT_XLOCALE */
