/*
 * Written by Steffen Nurpmeso.
 * Public domain.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "text.h"

struct tdat {
	char const *s;
	size_t l;
};

ATF_TC_BODY(txtcat_go, tc)
{
	static struct tdat const td[] = {
#define X(S)	{ S, sizeof(S) - 1 },
		X("")
		X("#")
		X("#1")		X("#12")	X("#123")	X("#12345")
		X("#123456")	X("#1234567")	X("#12345678")	X("#123456789")
		X("#1234567890") X("#1234567")	X("#12345678")	X("#123456789")
		/* With UTF-8 */
		X("#\303\244")
		X("#\303\244b")
		X("#\303\244bc")
		X("#\303\244bcd")
		X("#\303\244bcde")
		X("#\303\244bcdef")
		X("#\303\244bcdefg")
		X("#\303\244bcdefgh")
		X("#\303\244bcdefghi")
		X("#\303\244bcdefghij")
		X("#\303\244bcdefghijk")
		X("#\303\244bcdefghijkl")
		X("#\303\244bcdefghijklm")
		X("#\303\244bcdefghijklmn")
		X("#\303\244bcdefghijklmno")
		X("#\303\244bcdefghijklmn\303\266")
		X("#\303\244bcdefghijklmn\303\266p")
		X("#\303\244bcdefghijklmn\303\266pq")
		X("#\303\244bcdefghijklmn\303\266pqr")
		X("#\303\244bcdefghijklmn\303\266pqrs")
		X("#\303\244bcdefghijklmn\303\266pqrst")
		X("#\303\244bcdefghijklmn\303\266pqrstu")
#undef X
		{ NULL, 0 }
	};
	char dst[64];
	ssize_t (*f)(char *, char const *) = &txtcat;
	struct tdat const *x;
	ssize_t r;
	size_t i;

	for (x = td; x->s != NULL; ++x) {
		memset(dst, '@', sizeof dst);
		if (x->l > 0)
			dst[sizeof(dst)/2 + 0] = '#';
		dst[sizeof(dst)/2 + (x->l > 0)] = '\0';
		r = (*f)(dst + sizeof(dst)/2, x->s + (x->l > 0));
		if ((size_t)r != x->l)
			atf_tc_fail("txtcat() returned %zd, should be %zu\n",
				r, x->l);
		if (memcmp(dst + sizeof(dst)/2, x->s, x->l) != 0)
			atf_tc_fail("txtcat() copy failure (for %zu bytes)\n",
				x->l);
		for (i = 0; i < sizeof(dst)/2; ++i)
			if (dst[i] != '@')
				atf_tc_fail("txtcat() pre-buffer corruption "
					"(for %zu bytes, at %zu)\n", x->l, i);
		i += x->l;
		if (dst[i] != '\0')
			atf_tc_fail("txtcat() termination failure "
				"(for %zu bytes)\n", x->l);
		for (; ++i < sizeof dst;)
			if (dst[i] != '@')
				atf_tc_fail("txtcat() post-buffer corruption "
					"(for %zu bytes, at %zu)\n", x->l, i);
	}
}
