/*
 * Written by Steffen Nurpmeso.
 * Public domain.
 */

#include <locale.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "text.h"

struct tdat {
	int no;
	char const *s1;
	char const *s2;
	size_t n;
	ssize_t ret;
};

static struct tdat const	_td1_posix[] = {
	{ 0, "", "", 0, 0 },
	{ 1, "", "a", 0, -1 },
	{ 2, "a", "", 0, 1 },
	{ 3, "a", "a", 0, 0 },
	{ 4, "a", "b", 0, -1 },
	{ 5, "b", "a", 0, 1 },
	{ 6, "aa", "ab", 0, -2 },
	{ 7, "ab", "aa", 0, 2 },
	{ 8, "abracadabra", "abracadabra", 0, 0 },
	{ 9, "abracadabra", "abracadabrb", 0, -11 },
	{ 10, "abracadabrb", "abracadabra", 0, 11 },
	{ 11, "abracadabraa", "abracadabraa", 0, 0 },
	{ 12, "abracadabra", "abracadabraa", 0, -12 },
	{ 13, "abracadabraa", "abracadabra", 0, 12 },
#ifdef TEXT_SUPPORT_8BIT_C_LOCALE
	{ 14, "abracadabra\303\244", "abracadabra\303\244", 0, 0 },
	{ 15, "abracadabra\303\244", "abracadabra\303\266", 0, -13 },
	{ 16, "abracadabra\303\266", "abracadabra\303\244", 0, 13 },
	{ 17, "abracadabra\370\244", "abracadabra\370\244", 0, 0 },
	{ 18, "abracadabra\370\244", "abracadabra\370\266", 0, -13 },
	{ 19, "abracadabra\370\266", "abracadabra\370\244", 0, 13 },
#endif
	{ 0, NULL, NULL, 0, 0 }
},
				_td1_utf8[] = {
	{ 0, "", "", 0, 0 },
	{ 1, "", "a", 0, -1 },
	{ 2, "a", "", 0, 1 },
	{ 3, "a", "a", 0, 0 },
	{ 4, "a", "b", 0, -1 },
	{ 5, "b", "a", 0, 1 },
	{ 6, "aa", "ab", 0, -2 },
	{ 7, "ab", "aa", 0, 2 },
	{ 8, "abracadabra", "abracadabra", 0, 0 },
	{ 9, "abracadabra", "abracadabrb", 0, -11 },
	{ 10, "abracadabrb", "abracadabra", 0, 11 },
	{ 11, "abracadabraa", "abracadabraa", 0, 0 },
	{ 12, "abracadabra", "abracadabraa", 0, -12 },
	{ 13, "abracadabraa", "abracadabra", 0, 12 },
	{ 14, "abracadabra\303\244", "abracadabra\303\244", 0, 0 },
	{ 15, "abracadabra\303\244", "abracadabra\303\266", 0, -13 },
	{ 16, "abracadabra\303\266", "abracadabra\303\244", 0, 13 },
	{ 17, "abracadabra\370\244", "abracadabra\370\244", 0, 0 },
	{ 18, "abracadabra\370\244", "abracadabra\370\266", 0, 0 },
	{ 19, "abracadabra\370\266", "abracadabra\370\244", 0, 0 },
	{ 0, NULL, NULL, 0, 0 }
},
				_td2_posix[] = {
	{ 0, "", "", 0, 0 },
	{ 1, "", "a", 0, 0 },
	{ 2, "a", "", 0, 0 },
	{ 3, "", "a", 1, -1 },
	{ 4, "a", "", 1, 1 },
	{ 5, "a", "a", 1, 0 },
	{ 6, "a", "b", 1, -1 },
	{ 7, "b", "a", 1, 1 },
	{ 8, "aa", "ab", 2, -2 },
	{ 9, "ab", "aa", 2, 2 },
	{ 10, "abracadabra", "abracadabra", 11, 0 },
	{ 11, "abracadabra", "abracadabrb", 11, -11 },
	{ 12, "abracadabrb", "abracadabra", 11, 11 },
	{ 13, "abracadabraa", "abracadabraa", 12, 0 },
	{ 14, "abracadabra", "abracadabraa", 12, -12 },
	{ 15, "abracadabraa", "abracadabra", 12, 12 },
#ifdef TEXT_SUPPORT_8BIT_C_LOCALE
	{ 16, "abracadabra\303\244", "abracadabra\303\244", 13, 0 },
	{ 17, "abracadabra\303\244", "abracadabra\303\266", 13, -13 },
	{ 18, "abracadabra\303\266", "abracadabra\303\244", 13, 13 },
	{ 19, "abracadabra\303\244", "abracadabra\303\244", 11, 0 },
	{ 20, "abracadabra\303\244", "abracadabra\303\266", 12, 0 },
	{ 21, "abracadabra\370\244", "abracadabra\370\244", 13, 0 },
	{ 22, "abracadabra\370\244", "abracadabra\370\266", 13, -13 },
	{ 23, "abracadabra\370\266", "abracadabra\370\244", 13, 13 },
#endif
	{ 0, NULL, NULL, 0, 0 }
},
				_td2_utf8[] = {
	{ 0, "", "", 0, 0 },
	{ 1, "", "a", 0, 0 },
	{ 2, "a", "", 0, 0 },
	{ 3, "", "a", 1, -1 },
	{ 4, "a", "", 1, 1 },
	{ 5, "a", "a", 1, 0 },
	{ 6, "a", "b", 1, -1 },
	{ 7, "b", "a", 1, 1 },
	{ 8, "aa", "ab", 2, -2 },
	{ 9, "ab", "aa", 2, 2 },
	{ 10, "abracadabra", "abracadabra", 11, 0 },
	{ 11, "abracadabra", "abracadabrb", 11, -11 },
	{ 12, "abracadabrb", "abracadabra", 11, 11 },
	{ 13, "abracadabraa", "abracadabraa", 12, 0 },
	{ 14, "abracadabra", "abracadabraa", 12, -12 },
	{ 15, "abracadabraa", "abracadabra", 12, 12 },
	{ 16, "abracadabra\303\244", "abracadabra\303\244", 13, 0 },
	{ 17, "abracadabra\303\244", "abracadabra\303\266", 13, -13 },
	{ 18, "abracadabra\303\266", "abracadabra\303\244", 13, 13 },
	{ 19, "abracadabra\303\244", "abracadabra\303\244", 11, 0 },
	{ 20, "abracadabra\303\244", "abracadabra\303\266", 12, 0 },
	{ 21, "abracadabra\370\244", "abracadabra\370\244", 13, 0 },
	{ 22, "abracadabra\370\244", "abracadabra\370\266", 13, 0 },
	{ 23, "abracadabra\370\266", "abracadabra\370\244", 13, 0 },
	{ 0, NULL, NULL, 0, 0 }
},
				_tda_posix[] = {
	{ 1, "a", "A", 1, 0 },
	{ 2, "a", "B", 1, -1 },
	{ 3, "B", "a", 1, 1 },
	{ 4, "Aa", "aB", 2, -2 },
	{ 5, "aB", "Aa", 2, 2 },
	{ 6, "aBrAcadAbra", "AbracadabrA", 11, 0 },
	{ 7, "abracadabra", "abracadabrB", 11, -11 },
	{ 8, "abracadabrB", "abracadabra", 11, 11 },
	{ 9, "abracadabrAa", "abracadabraA", 12, 0 },
#ifdef TEXT_SUPPORT_8BIT_C_LOCALE
	{ 10, "abracadabra\303\266", "abracadabra\303\226", 13, 13 },
	{ 11, "abracadabra\303\226", "abracadabra\303\266", 13, -13 },
#endif
	{ 0, NULL, NULL, 0, 0 }
},
				_tda_utf8[] = {
	{ 1, "a", "A", 1, 0 },
	{ 2, "a", "B", 1, -1 },
	{ 3, "B", "a", 1, 1 },
	{ 4, "Aa", "aB", 2, -2 },
	{ 5, "aB", "Aa", 2, 2 },
	{ 6, "aBrAcadAbra", "AbracadabrA", 11, 0 },
	{ 7, "abracadabra", "abracadabrB", 11, -11 },
	{ 8, "abracadabrB", "abracadabra", 11, 11 },
	{ 9, "abracadabrAa", "abracadabraA", 12, 0 },
#ifdef TEXT_SUPPORT_8BIT_C_LOCALE
	{ 10, "abracadabra\303\266", "abracadabra\303\226", 13, 0 },
	{ 11, "abracadabra\303\226", "abracadabra\303\266", 13, 0 },
#endif
	{ 0, NULL, NULL, 0, 0 }
};

static char const *	_ts;
static ssize_t	(*	_tptf)(char const *, char const *, size_t);
#ifdef TEXT_SUPPORT_XLOCALE
static locale_t		_lc;
#endif

static void	_t1(int tno, struct tdat const *);

static void
_t1(int tno, struct tdat const *td)
{
	struct tdat const *x;
	ssize_t r;

	for (x = td; x->s1 != NULL; ++x) {
		r = (*_tptf)(x->s1, x->s2, x->n);
		if (r != x->ret)
			atf_tc_fail("%d.%d %s(): got %zd, should be %zd\n",
				tno, x->no, _ts, r, x->ret);
	}
}

static ssize_t
_tgo(char const *s1, char const *s2, size_t n)
{ (void)n; return txtcmp(s1, s2); }

ATF_TC_BODY(txtcmp_go, tc)
{
	_tptf = &_tgo;

	_ts = "POSIX:txtcmp";
	if (setlocale(LC_CTYPE, "POSIX") == 0)
		atf_tc_fail("1. %s: cannot set locale\n", _ts);
	_t1(1, _td1_posix);

	_ts = "en_USSR.UTF-8:txtcmp";
	if (setlocale(LC_CTYPE, "en_US.UTF-8") == 0)
		atf_tc_fail("1. %s: cannot set locale\n", _ts);
	_t1(2, _td1_utf8);
}

#ifdef TEXT_SUPPORT_XLOCALE
static ssize_t
_tgo_l(char const *s1, char const *s2, size_t n)
{ (void)n; return txtcmp_l(s1, s2, _lc); }

ATF_TC_BODY(txtcmp_l_go, tc)
{
	_tptf = &_tgo_l;

	_ts = "POSIX:txtcmp_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "POSIX", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("2. %s: cannot newlocale()\n", _ts);
	_t1(3, _td1_posix);

	_ts = "en_USSR.UTF-8:txtcmp_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "en_US.UTF-8", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("2. %s: cannot newlocale()\n", _ts);
	_t1(4, _td1_utf8);
}
#endif /* TEXT_SUPPORT_XLOCALE */

static ssize_t
_tngo(char const *s1, char const *s2, size_t n)
{ return txtncmp(s1, s2, n); }

ATF_TC_BODY(txtncmp_go, tc)
{
	_tptf = &_tngo;

	_ts = "POSIX:txtncmp";
	if (setlocale(LC_CTYPE, "POSIX") == 0)
		atf_tc_fail("3. %s: cannot set locale\n", _ts);
	_t1(5, _td2_posix);

	_ts = "en_USSR.UTF-8:txtncmp";
	if (setlocale(LC_CTYPE, "en_US.UTF-8") == 0)
		atf_tc_fail("3. %s: cannot set locale\n", _ts);
	_t1(6, _td2_utf8);
}

#ifdef TEXT_SUPPORT_XLOCALE
static ssize_t
_tngo_l(char const *s1, char const *s2, size_t n)
{ return txtncmp_l(s1, s2, n, _lc); }

ATF_TC_BODY(txtncmp_l_go, tc)
{
	_tptf = &_tngo_l;

	_ts = "POSIX:txtncmp_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "POSIX", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("4. %s: cannot newlocale()\n", _ts);
	_t1(7, _td2_posix);

	_ts = "en_USSR.UTF-8:txtncmp_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "en_US.UTF-8", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("4. %s: cannot newlocale()\n", _ts);
	_t1(8, _td2_utf8);
}
#endif /* TEXT_SUPPORT_XLOCALE */

static ssize_t
_tcasego(char const *s1, char const *s2, size_t n)
{ (void)n; return txtcasecmp(s1, s2); }

ATF_TC_BODY(txtcasecmp_go, tc)
{
	_tptf = &_tcasego;

	_ts = "POSIX:txtcasecmp";
	if (setlocale(LC_CTYPE, "POSIX") == 0)
		atf_tc_fail("1. %s: cannot set locale\n", _ts);
	_t1(9, _td1_posix);
	_t1(10, _tda_posix);

	_ts = "en_USSR.UTF-8:txtcasecmp";
	if (setlocale(LC_CTYPE, "en_US.UTF-8") == 0)
		atf_tc_fail("1. %s: cannot set locale\n", _ts);
	_t1(11, _td1_utf8);
	_t1(12, _tda_utf8);
}

#ifdef TEXT_SUPPORT_XLOCALE
static ssize_t
_tcasego_l(char const *s1, char const *s2, size_t n)
{ (void)n; return txtcasecmp_l(s1, s2, _lc); }

ATF_TC_BODY(txtcasecmp_l_go, tc)
{
	_tptf = &_tcasego_l;

	_ts = "POSIX:txtcasecmp_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "POSIX", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("2. %s: cannot newlocale()\n", _ts);
	_t1(13, _td1_posix);
	_t1(14, _tda_posix);

	_ts = "en_USSR.UTF-8:txtcasecmp_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "en_US.UTF-8", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("2. %s: cannot newlocale()\n", _ts);
	_t1(15, _td1_utf8);
	_t1(16, _tda_utf8);
}
#endif /* TEXT_SUPPORT_XLOCALE */

static ssize_t
_tncasego(char const *s1, char const *s2, size_t n)
{ return txtncasecmp(s1, s2, n); }

ATF_TC_BODY(txtncasecmp_go, tc)
{
	_tptf = &_tncasego;

	_ts = "POSIX:txtncasecmp";
	if (setlocale(LC_CTYPE, "POSIX") == 0)
		atf_tc_fail("3. %s: cannot set locale\n", _ts);
	_t1(13, _td2_posix);

	_ts = "en_USSR.UTF-8:txtncasecmp";
	if (setlocale(LC_CTYPE, "en_US.UTF-8") == 0)
		atf_tc_fail("3. %s: cannot set locale\n", _ts);
	_t1(14, _td2_utf8);
}

#ifdef TEXT_SUPPORT_XLOCALE
static ssize_t
_tncasego_l(char const *s1, char const *s2, size_t n)
{ return txtncasecmp_l(s1, s2, n, _lc); }

ATF_TC_BODY(txtncasecmp_l_go, tc)
{
	_tptf = &_tncasego_l;

	_ts = "POSIX:txtncasecmp_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "POSIX", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("4. %s: cannot newlocale()\n", _ts);
	_t1(15, _td2_posix);

	_ts = "en_USSR.UTF-8:txtncasecmp_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "en_US.UTF-8", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("4. %s: cannot newlocale()\n", _ts);
	_t1(16, _td2_utf8);
}
#endif /* TEXT_SUPPORT_XLOCALE */
