/*
 * Written by Steffen Nurpmeso.
 * Public domain.
 */

#include <locale.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "text.h"

struct tdat {
	int no;
	char const *s;
	size_t n;
	ssize_t len;
	size_t charcnt;
};

static struct tdat const	_td1_posix[] = {
	{ 0, "", 0, 0 , 0 },
	{ 1, "a", 1, 1, 1 },
	{ 2, "A", 1, 1, 1 },
	{ 3, "\t", 1, 1, 1 },
#ifdef TEXT_SUPPORT_8BIT_C_LOCALE
	{ 4, "\303\244", 2, 2, 2 },
	{ 5, "\370\244", 2, 2, 2 },
	{ 6, "\341\240\200", 3, 3, 3 },
	{ 7, "\361\240\177", 3, 3, 3 },
#endif
	{ 8, "a\t", 2, 2, 2 },
	{ 9, "abc\t", 4, 4, 4 },
	{ 10, "gamma", 5, 5, 5 },
#ifdef TEXT_SUPPORT_8BIT_C_LOCALE
	{ 11, "gamma\303\244", 6, 6, 6 },
	{ 12, "gamma\303\244", 7, 7, 7 },
	{ 13, "gamma\341\240\200", 7, 7, 7 },
	{ 14, "gamma\341\240\200", 8, 8, 8 },
	{ 15, "gamma\361\240\200", 8, 8, 8 },
#endif
	{ 0, NULL, 0, 0, 0 }
},
				_td1_utf8[] = {
	{ 0, "", 0, 0, 0 },
	{ 1, "a", 1, 1, 1 },
	{ 2, "A", 1, 1, 1 },
	{ 3, "\t", 1, 1, 1 },
	{ 4, "\303\244", 2, 2, 1 },
	{ 5, "\370\244", 2, -1, 0 },
	{ 6, "\341\240\200", 3, 3, 1 },
	{ 7, "\361\240\200", 3, -1, 0 },
	{ 8, "a\t", 2, 2, 2 },
	{ 9, "abc\t", 4, 4, 4 },
	{ 10, "gamma", 5, 5, 5 },
	{ 11, "gamma\303\244", 6, -6, 5 },
	{ 12, "gamma\303\244", 7, 7, 6 },
	{ 13, "gamma\341\240\200", 7, -6, 5 },
	{ 14, "gamma\341\240\200", 8, 8, 6 },
	{ 15, "gamma\361\240\200", 8, -6, 5 },
	{ 0, NULL, 0, 0, 0 }
};

static char const *	_ts;
static ssize_t	(*	_tptf)(char const *, size_t, size_t *);
#ifdef TEXT_SUPPORT_XLOCALE
static locale_t		_lc;
#endif

static void	_t1(int tno, struct tdat const *, int nolen);

static void
_t1(int tno, struct tdat const *td, int nolen)
{
	struct tdat const *x;
	size_t charcnt;

	for (x = td; x->s != NULL; ++x) {
		ssize_t r;
		if (nolen && strlen(x->s) != x->n)
			continue;
		r = (*_tptf)(x->s, x->n, &charcnt);
		if (r != x->len)
			atf_tc_fail("%d.%d %s(): got %zd, should be %zd\n",
				tno, x->no, _ts, r, x->len);
		if (charcnt != x->charcnt)
			atf_tc_fail("%d.%d %s(): charcnt is %zu, should %zu\n",
				tno, x->no, _ts, charcnt, x->charcnt);
	}
}

static ssize_t
_tgo(char const *s, size_t n, size_t *ccnt)
{ (void)n; return txtlen(s, ccnt); }

ATF_TC_BODY(txtlen_go, tc)
{
	_tptf = &_tgo;

	_ts = "POSIX:txtlen";
	if (setlocale(LC_CTYPE, "POSIX") == 0)
		atf_tc_fail("1. %s: cannot set locale\n", _ts);
	_t1(1, _td1_posix, 1);

	_ts = "en_USSR.UTF-8:txtlen";
	if (setlocale(LC_CTYPE, "en_US.UTF-8") == 0)
		atf_tc_fail("1. %s: cannot set locale\n", _ts);
	_t1(2, _td1_utf8, 1);
}

#ifdef TEXT_SUPPORT_XLOCALE
static ssize_t
_tgo_l(char const *s, size_t n, size_t *ccnt)
{ (void)n; return txtlen_l(s, ccnt, _lc); }

ATF_TC_BODY(txtlen_l_go, tc)
{
	_tptf = &_tgo_l;

	_ts = "POSIX:txtlen_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "POSIX", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("2. %s: cannot newlocale()\n", _ts);
	_t1(3, _td1_posix, 1);

	_ts = "en_USSR.UTF-8:txtlen_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "en_US.UTF-8", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("2. %s: cannot newlocale()\n", _ts);
	_t1(4, _td1_utf8, 1);
}
#endif /* TEXT_SUPPORT_XLOCALE */

static ssize_t
_tngo(char const *s, size_t n, size_t *ccnt)
{ return txtnlen(s, n, ccnt); }

ATF_TC_BODY(txtnlen_go, tc)
{
	_tptf = &_tngo;

	_ts = "POSIX:txtnlen";
	if (setlocale(LC_CTYPE, "POSIX") == 0)
		atf_tc_fail("1. %s: cannot set locale\n", _ts);
	_t1(5, _td1_posix, 0);

	_ts = "en_USSR.UTF-8:txtnlen";
	if (setlocale(LC_CTYPE, "en_US.UTF-8") == 0)
		atf_tc_fail("1. %s: cannot set locale\n", _ts);
	_t1(6, _td1_utf8, 0);
}

#ifdef TEXT_SUPPORT_XLOCALE
static ssize_t
_tngo_l(char const *s, size_t n, size_t *ccnt)
{ return txtnlen_l(s, n, ccnt, _lc); }

ATF_TC_BODY(txtnlen_l_go, tc)
{
	_tptf = &_tngo_l;

	_ts = "POSIX:txtnlen_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "POSIX", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("2. %s: cannot newlocale()\n", _ts);
	_t1(7, _td1_posix, 0);

	_ts = "en_USSR.UTF-8:txtnlen_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "en_US.UTF-8", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("2. %s: cannot newlocale()\n", _ts);
	_t1(8, _td1_utf8, 0);
}
#endif /* TEXT_SUPPORT_XLOCALE */
