/*
 * Written by Steffen Nurpmeso.
 * Public domain.
 */

#include <locale.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "text.h"

struct tdat {
	int no;
	char const *s;
	size_t off;
	size_t size;
	size_t dlfinal;
	ssize_t ret;
};

static struct tdat const	_td1_posix[] = {
	{ 0, "#DEAR", 10, 0, 0, 4 },
	{ 1, "#DEAR", 10, 1, 1, 5 },
	{ 2, "#DEAR", 10, 3, 2, 5 },
	{ 3, "#DEAR", 10, 4, 3, 5 },
	{ 4, "#DEAR", 10, 5, 4, 5 },
	{ 5, "#DEAR", 10, 6, 5, 5 },
#ifdef TEXT_SUPPORT_8BIT_C_LOCALE
	{ 6, "#DE\303\204R", 10, 5, 4, 6 },
	{ 7, "#DE\303\204R", 10, 6, 5, 6 },
	{ 8, "#DE\303\204R", 10, 7, 6, 6 },
	{ 9, "#DE\303\204\303\226R", 10, 6, 5, 8 },
	{ 10, "#DE\303\204\303\226R", 10, 7, 6, 8 },
	{ 11, "#DE\303\204\303\226R", 10, 8, 7, 8 },
	{ 12, "#DE\303\204\303\226R", 10, 9, 8, 8 },
	{ 13, "#MO\341\240\200R", 10, 4, 3, 7 },
	{ 14, "#MO\341\240\200R", 10, 5, 4, 7 },
	{ 15, "#MO\341\240\200R", 10, 6, 5, 7 },
	{ 16, "#MO\341\240\200R", 10, 7, 6, 7 },
	{ 17, "#MO\341\240\200R", 10, 8, 7, 7 },
	{ 18, "#MO\361\240\200R", 10, 8, 7, 7 },
	{ 19, "#MO\341\240\200R\303\204", 10, 8, 7, 9 },
	{ 20, "#MO\341\240\200R\303\204", 10, 9, 8, 9 },
	{ 21, "#MO\341\240\200R\303\204", 10, 10, 9, 9 },
	{ 22, "#MO\341\240\200R\341\240\200R", 10, 9, 8, 11 },
	{ 23, "#MO\341\240\200R\341\240\200R", 10, 10, 9, 11 },
	{ 24, "#MO\341\240\200R\341\240\200R", 10, 11, 10, 11 },
	{ 25, "#MO\341\240\200R\341\240\200R", 10, 12, 11, 11 },
	{ 26, "#MO\341\240\200R\361\240\200R", 10, 12, 11, 11 },
#endif
	{ 0, NULL, 0, 0, 0, 0 }
},
				_td1_utf8[] = {
	{ 0, "#DEAR", 10, 0, 0, 4 },
	{ 1, "#DEAR", 10, 1, 1, 5 },
	{ 2, "#DEAR", 10, 3, 2, 5 },
	{ 3, "#DEAR", 10, 4, 3, 5 },
	{ 4, "#DEAR", 10, 5, 4, 5 },
	{ 5, "#DEAR", 10, 6, 5, 5 },
	{ 6, "#DE\303\204R", 10, 5, 3, 6 },
	{ 7, "#DE\303\204R", 10, 6, 5, 6 },
	{ 8, "#DE\303\204R", 10, 7, 6, 6 },
	{ 9, "#DE\303\204\303\226R", 10, 6, 5, 8 },
	{ 10, "#DE\303\204\303\226R", 10, 7, 5, 8 },
	{ 11, "#DE\303\204\303\226R", 10, 8, 7, 8 },
	{ 12, "#DE\303\204\303\226R", 10, 9, 8, 8 },
	{ 13, "#MO\341\240\200R", 10, 4, 3, 7 },
	{ 14, "#MO\341\240\200R", 10, 5, 3, 7 },
	{ 15, "#MO\341\240\200R", 10, 6, 3, 7 },
	{ 16, "#MO\341\240\200R", 10, 7, 6, 7 },
	{ 17, "#MO\341\240\200R", 10, 8, 7, 7 },
	{ 18, "#MO\361\240\200R", 10, 8, 3, -3 },
	{ 19, "#MO\341\240\200R\303\204", 10, 8, 7, 9 },
	{ 20, "#MO\341\240\200R\303\204", 10, 9, 7, 9 },
	{ 21, "#MO\341\240\200R\303\204", 10, 10, 9, 9 },
	{ 22, "#MO\341\240\200R\341\240\200R", 10, 9, 7, 11 },
	{ 23, "#MO\341\240\200R\341\240\200R", 10, 10, 7, 11 },
	{ 24, "#MO\341\240\200R\341\240\200R", 10, 11, 10, 11 },
	{ 25, "#MO\341\240\200R\341\240\200R", 10, 12, 11, 11 },
	{ 26, "#MO\341\240\200R\361\240\200R", 10, 12, 7, -7 },
	{ 0, NULL, 0, 0, 0, 0 }
};

static char const *	_ts;
static ssize_t	(*	_tptf)(char *, char const *, size_t, size_t *);
#ifdef TEXT_SUPPORT_XLOCALE
static locale_t		_lc;
#endif

static void	_t1(struct tdat const *);

static void
_t1(struct tdat const *td)
{
	char dst[64];
	struct tdat const *x;
	ssize_t r;
	size_t dlfinal, i;

	for (x = td; x->s != NULL; ++x) {
		memset(dst, '@', sizeof dst);
		if (x->size > 0)
			dst[x->off] = '#';
		dst[x->off + (x->size > 0)] = '\0';
		r = (*_tptf)(dst + x->off, x->s + 1, x->size, &dlfinal);
		if (r != x->ret)
			atf_tc_fail("1.%d %s(): got %zd, should be %zd\n",
				x->no, _ts, r, x->ret);
		if (dlfinal != x->dlfinal)
			atf_tc_fail("1.%d %s(): `dstlen_final' is %zu, "
				"should be %zu\n",
				x->no, _ts, dlfinal, x->dlfinal);
		if (memcmp(dst + x->off, x->s, x->dlfinal) != 0)
			atf_tc_fail("1.%d %s(): copy failure\n", x->no, _ts);
		for (i = 0; i < x->off; ++i)
			if (dst[i] != '@')
				atf_tc_fail("1.%d %s(): pre-buf "
					"corruption (at %zu)\n", x->no, _ts, i);
		i += x->dlfinal;
		if (x->dlfinal > 0 && dst[i] != '\0')
			atf_tc_fail("1.%d %s(): termination failure\n",
				x->no, _ts);
		for (; ++i < sizeof(dst);)
			if (dst[i] != '@')
				atf_tc_fail("1.%d %s(): post-buf "
					"corruption (at %zu)\n", x->no, _ts, i);
	}
}

static ssize_t
_tgo(char *dst, char const *src, size_t n, size_t *dlfinal)
{ return txtllcat(dst, n, src, strlen(src), dlfinal); }

ATF_TC_BODY(txtllcat_go, tc)
{
	_tptf = &_tgo;

	_ts = "POSIX:txtllcat";
	if (setlocale(LC_CTYPE, "POSIX") == 0)
		atf_tc_fail("1. %s: cannot set locale\n", _ts);
	_t1(_td1_posix);

	_ts = "en_USSR.UTF-8:txtllcat";
	if (setlocale(LC_CTYPE, "en_US.UTF-8") == 0)
		atf_tc_fail("1. %s: cannot set locale\n", _ts);
	_t1(_td1_utf8);
}

#ifdef TEXT_SUPPORT_XLOCALE
static ssize_t
_tgo_l(char *dst, char const *src, size_t n, size_t *dlfinal)
{ return txtllcat_l(dst, n, src, strlen(src), dlfinal, _lc); }

ATF_TC_BODY(txtllcat_l_go, tc)
{
	_tptf = &_tgo_l;

	_ts = "POSIX:txtllcat_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "POSIX", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("2. %s: cannot newlocale()\n", _ts);
	_t1(_td1_posix);

	_ts = "en_USSR.UTF-8:txtllcat_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "en_US.UTF-8", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("2. %s: cannot newlocale()\n", _ts);
	_t1(_td1_utf8);
}
#endif /* TEXT_SUPPORT_XLOCALE */
