/*
 * Written by Steffen Nurpmeso.
 * Public domain.
 */

#include <locale.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "text.h"

struct tdat {
	int no;
	char const *s;
	size_t off;
	size_t size;
	ssize_t ret;
};

static struct tdat const	_td1_posix[] = {
	{ 0, "DEAR", 10, 0, 0 },
	{ 1, "DEAR", 10, 3, 3 },
	{ 2, "DEAR", 10, 4, 4 },
	{ 3, "DEAR", 10, 5, 4 },
#ifdef TEXT_SUPPORT_8BIT_C_LOCALE
	{ 4, "DE\303\204R", 10, 5, 5 },
	{ 5, "DE\303\204\303\226R", 10, 5, 5 },
	{ 6, "DE\303\204\303\226R", 10, 6, 6 },
	{ 7, "DE\303\204\303\226R", 10, 7, 7 },
	{ 8, "DE\303\204\303\226R", 10, 8, 7 },
	{ 9, "MO\341\240\200R", 10, 3, 3 },
	{ 10, "MO\341\240\200R", 10, 4, 4 },
	{ 11, "MO\341\240\200R", 10, 5, 5 },
	{ 12, "MO\341\240\200R", 10, 6, 6 },
	{ 13, "MO\361\240\200R", 10, 7, 6 },
	{ 14, "MO\341\240\200R\303\204", 10, 7, 7 },
	{ 15, "MO\341\240\200R\303\204", 10, 8, 8 },
	{ 16, "MO\341\240\200R\303\204", 10, 9, 8 },
	{ 17, "MO\341\240\200R\361\240\200", 10, 7, 7 },
	{ 18, "MO\341\240\200R\361\240\200", 10, 8, 8 },
	{ 19, "MO\341\240\200R\361\240\200", 10, 9, 9 },
	{ 20, "MO\341\240\200R\361\240\200", 10, 10, 9 },
#endif
	{ 0, NULL, 0, 0, 0 }
},
				_td1_utf8[] = {
	{ 0, "DEAR", 10, 0, 0 },
	{ 1, "DEAR", 10, 3, 3 },
	{ 2, "DEAR", 10, 4, 4 },
	{ 3, "DEAR", 10, 5, 4 },
	{ 4, "DE\303\204R", 10, 5, 5 },
	{ 5, "DE\303\204\303\226R", 10, 5, 4 },
	{ 6, "DE\303\204\303\226R", 10, 6, 6 },
	{ 7, "DE\303\204\303\226R", 10, 7, 7 },
	{ 8, "DE\303\204\303\226R", 10, 8, 7 },
	{ 9, "MO\341\240\200R", 10, 3, 2 },
	{ 10, "MO\341\240\200R", 10, 4, 2 },
	{ 11, "MO\341\240\200R", 10, 5, 5 },
	{ 12, "MO\341\240\200R", 10, 6, 6 },
	{ 13, "MO\361\240\200R", 10, 7, -3 },
	{ 14, "MO\341\240\200R\303\204", 10, 7, 6 },
	{ 15, "MO\341\240\200R\303\204", 10, 8, 8 },
	{ 16, "MO\341\240\200R\303\204", 10, 9, 8 },
	{ 17, "MO\341\240\200R\361\240\200", 10, 6, 6 },
	{ 18, "MO\341\240\200R\361\240\200", 10, 7, -7 },
	{ 19, "MO\341\240\200R\361\240\200", 10, 8, -7 },
	{ 20, "MO\341\240\200R\361\240\200", 10, 10, -7 },
	{ 0, NULL, 0, 0, 0 }
};

static char const *	_ts;
static ssize_t	(*	_tptf)(char *, char const *, size_t);
#ifdef TEXT_SUPPORT_XLOCALE
static locale_t		_lc;
#endif

static void	_t1(struct tdat const *);

static void
_t1(struct tdat const *td)
{
	char dst[64];
	struct tdat const *x;
	ssize_t r;
	size_t i;

	for (x = td; x->s != NULL; ++x) {
		memset(dst, '@', sizeof dst);
		r = (*_tptf)(dst + x->off, x->s, x->size);
		if (r != x->ret)
			atf_tc_fail("1.%d %s(): got %zd, should be %zd\n",
				x->no, _ts, r, x->ret);
		if (r < 0)
			r = -(r + 1);
		if (memcmp(dst + x->off, x->s, r) != 0)
			atf_tc_fail("1.%d %s(): copy failure\n", x->no, _ts);
		for (i = 0; i < x->off; ++i)
			if (dst[i] != '@')
				atf_tc_fail("1.%d %s(): pre-buf "
					"corruption (at %zu)\n", x->no, _ts, i);
		i += r;
		if (x->size > (size_t)r) {
			for (; x->size > (size_t)r; ++i, ++r)
				if (dst[i] != '\0')
					atf_tc_fail("1.%d %s(): termination "
						"failure 1 (%zu)\n",
						x->no, _ts, i);
		} else if (dst[i] == '\0')
			atf_tc_fail("1.%d %s(): termination failure 2\n",
				x->no, _ts);
		for (; ++i < sizeof(dst);)
			if (dst[i] != '@')
				atf_tc_fail("1.%d %s(): post-buf "
					"corruption (at %zu)\n", x->no, _ts, i);
	}
}

static ssize_t
_tgo(char *dst, char const *src, size_t n)
{ return txtncpy(dst, src, n); }

ATF_TC_BODY(txtncpy_go, tc)
{
	_tptf = &_tgo;

	_ts = "POSIX:txtncpy";
	if (setlocale(LC_CTYPE, "POSIX") == 0)
		atf_tc_fail("1. %s: cannot set locale\n", _ts);
	_t1(_td1_posix);

	_ts = "en_USSR.UTF-8:txtncpy";
	if (setlocale(LC_CTYPE, "en_US.UTF-8") == 0)
		atf_tc_fail("1. %s: cannot set locale\n", _ts);
	_t1(_td1_utf8);
}

#ifdef TEXT_SUPPORT_XLOCALE
static ssize_t
_tgo_l(char *dst, char const *src, size_t n)
{ return txtncpy_l(dst, src, n, _lc); }

ATF_TC_BODY(txtncpy_l_go, tc)
{
	_tptf = &_tgo_l;

	_ts = "POSIX:txtncpy_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "POSIX", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("2. %s: cannot newlocale()\n", _ts);
	_t1(_td1_posix);

	_ts = "en_USSR.UTF-8:txtncpy_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "en_US.UTF-8", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("2. %s: cannot newlocale()\n", _ts);
	_t1(_td1_utf8);
}
#endif /* TEXT_SUPPORT_XLOCALE */
