/*
 * Written by Steffen Nurpmeso.
 * Public domain.
 */

#include <locale.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "text.h"

struct tdat {
	int no;
	char const *s1;
	char const *s2;
	size_t s1len;
	size_t s2len;
	ssize_t ret;
	ssize_t revret;
};

static struct tdat const	_td1_posix[] = {
	{ 0, "", "", 0, 0, 0 , 0 },
	{ 1, "", "a", 0, 1, -1, -1 },
	{ 2, "a", "", 1, 0, 0, 0 },
	{ 3, "a", "a", 1, 1, 0, 0 },
	{ 4, "ab", "b", 2, 1, 1, 1 },
	{ 5, "aaab", "ab", 4, 2, 2, 2 },
	{ 6, "aaab", "ab", 3, 2, -1, -1 },
#ifdef TEXT_SUPPORT_8BIT_C_LOCALE
	{ 7, "cba\303\244bc", "cba\303\244bc", 7, 7, 0, 0 },
	{ 8, "cba\303\244bc", "\303\244bc", 7, 4, 3, 3 },
	{ 9, "cba\303\244bc", "\244bc", 7, 3, 4, 4 },
	{ 10, "cba\303\244bc\303\244bc", "\303\244bc", 11, 4, 3, 7 },
	{ 11, "cba\303\244bc\303\244bc", "\244bc", 11, 3, 4, 8 },
#endif
	{ 0, NULL, NULL, 0, 0, 0, 0 }
},
				_td1_utf8[] = {
	{ 0, "", "", 0, 0, 0, 0 },
	{ 1, "", "a", 0, 1, -1, -1 },
	{ 2, "a", "", 1, 0, 0, 0 },
	{ 3, "a", "a", 1, 1, 0, 0 },
	{ 4, "ab", "b", 2, 1, 1, 1 },
	{ 5, "aaab", "ab", 4, 2, 2, 2 },
	{ 6, "aaab", "ab", 3, 2, -1, -1 },
	{ 7, "cba\303\244bc", "cba\303\244bc", 7, 7, 0, 0 },
	{ 8, "cba\303\244bc", "\303\244bc", 7, 4, 3, 3 },
	{ 9, "cba\303\244bc", "\244bc", 7, 3, -1, -1 },
	{ 10, "cba\303\244bc\303\244bc", "\303\244bc", 11, 4, 3, 7 },
	{ 11, "cba\303\244bc\303\244bc", "\244bc", 11, 3, -1, -1 },
	{ 0, NULL, NULL, 0, 0, 0, 0 }
},
				_td2_posix[] = {
	{ 0, "", "", 0, 0, 0, 0 },
	{ 1, "", "a", 0, 1, -1, -1 },
	{ 2, "a", "", 1, 0, 0, 0 },
	{ 3, "a", "a", 1, 1, 0, 0 },
	{ 4, "ab", "B", 2, 1, 1, 1 },
	{ 5, "aaab", "AB", 4, 2, 2, 2 },
	{ 6, "aaab", "AB", 3, 2, -1, -1 },
#ifdef TEXT_SUPPORT_8BIT_C_LOCALE
	{ 7, "cba\303\266bc", "cba\303\244bc", 7, 7, -1, -1 },
	{ 8, "cba\303\244bc", "\303\204bc", 7, 4, -1, -1 },
	{ 9, "cba\303\244bc", "\244bc", 7, 3, 4, 4 },
	{ 10, "cba\303\244bc\303\244bc", "\303\204bc", 11, 4, -1, -1 },
	{ 11, "cba\303\244bc\303\244bc", "\244bc", 11, 3, 4, 8 },
#endif
	{ 0, NULL, NULL, 0, 0, 0, 0 }
},
				_td2_utf8[] = {
	{ 0, "", "", 0, 0, 0, 0 },
	{ 1, "", "a", 0, 1, -1, -1 },
	{ 2, "a", "", 1, 0, 0, 0 },
	{ 3, "a", "a", 1, 1, 0, 0 },
	{ 4, "ab", "B", 2, 1, 1, 1 },
	{ 5, "aaab", "AB", 4, 2, 2, 2 },
	{ 6, "aaab", "AB", 3, 2, -1, -1 },
	{ 7, "cba\303\204bc", "cba\303\244bc", 7, 7, 0, 0 },
	{ 8, "cba\303\244bc", "\303\204bc", 7, 4, 3, 3 },
	{ 9, "cba\303\244bc", "\244bc", 7, 3, -1, -1 },
	{ 10, "cba\303\244bc\303\244bc", "\303\204bc", 11, 4, 3, 7 },
	{ 11, "cba\303\244bc\303\244bc", "\244bc", 11, 3, -1, -1 },
	{ 0, NULL, NULL, 0, 0, 0, 0 }
};

static char const *	_ts;
static char const * (*	_tptf)(char const *, size_t, char const *, size_t);
#ifdef TEXT_SUPPORT_XLOCALE
static locale_t		_lc;
#endif

static void	_t1(int tno, struct tdat const *td, int nolen, int rev);

static void
_t1(int tno, struct tdat const *td, int nolen, int rev)
{
	struct tdat const *x;
	ssize_t r;

	for (x = td; x->s1 != NULL; ++x) {
		char const *s;
		if (nolen && (strlen(x->s1) != x->s1len ||
				strlen(x->s2) != x->s2len))
			continue;
		s = (*_tptf)(x->s1, x->s1len, x->s2, x->s2len);
		if (s == NULL)
			r = -1;
		else
			r = (ssize_t)(s - x->s1);
		if (r != (rev ? x->revret : x->ret))
			atf_tc_fail("%d.%d %s(): got %zd, should be %zd\n",
				tno, x->no, _ts, r, (rev ? x->revret : x->ret));
	}
}

static char const *
_tgo(char const *s1, size_t s1len, char const *s2, size_t s2len)
{ (void)s1len; (void)s2len; return txtstr(s1, s2); }

ATF_TC_BODY(txtstr_go, tc)
{
	_tptf = &_tgo;

	_ts = "POSIX:txtstr";
	if (setlocale(LC_CTYPE, "POSIX") == 0)
		atf_tc_fail("1. %s: cannot set locale\n", _ts);
	_t1(1, _td1_posix, 1, 0);

	_ts = "en_USSR.UTF-8:txtstr";
	if (setlocale(LC_CTYPE, "en_US.UTF-8") == 0)
		atf_tc_fail("1. %s: cannot set locale\n", _ts);
	_t1(2, _td1_utf8, 1, 0);
}

#ifdef TEXT_SUPPORT_XLOCALE
static char const *
_tgo_l(char const *s1, size_t s1len, char const *s2, size_t s2len)
{ (void)s1len; (void)s2len; return txtstr_l(s1, s2, _lc); }

ATF_TC_BODY(txtstr_l_go, tc)
{
	_tptf = &_tgo_l;

	_ts = "POSIX:txtstr_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "POSIX", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("2. %s: cannot newlocale()\n", _ts);
	_t1(3, _td1_posix, 1, 0);

	_ts = "en_USSR.UTF-8:txtstr_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "en_US.UTF-8", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("2. %s: cannot newlocale()\n", _ts);
	_t1(4, _td1_utf8, 1, 0);
}
#endif /* TEXT_SUPPORT_XLOCALE */

static char const *
_tllgo(char const *s1, size_t s1len, char const *s2, size_t s2len)
{ return txtllstr(s1, s1len, s2, s2len); }

ATF_TC_BODY(txtllstr_go, tc)
{
	_tptf = &_tllgo;

	_ts = "POSIX:txtllstr";
	if (setlocale(LC_CTYPE, "POSIX") == 0)
		atf_tc_fail("3. %s: cannot set locale\n", _ts);
	_t1(5, _td1_posix, 0, 0);

	_ts = "en_USSR.UTF-8:txtllstr";
	if (setlocale(LC_CTYPE, "en_US.UTF-8") == 0)
		atf_tc_fail("3. %s: cannot set locale\n", _ts);
	_t1(6, _td1_utf8, 0, 0);
}

#ifdef TEXT_SUPPORT_XLOCALE
static char const *
_tllgo_l(char const *s1, size_t s1len, char const *s2, size_t s2len)
{ return txtllstr_l(s1, s1len, s2, s2len, _lc); }

ATF_TC_BODY(txtllstr_l_go, tc)
{
	_tptf = &_tllgo_l;

	_ts = "POSIX:txtllstr_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "POSIX", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("4. %s: cannot newlocale()\n", _ts);
	_t1(7, _td1_posix, 0, 0);

	_ts = "en_USSR.UTF-8:txtllstr_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "en_US.UTF-8", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("4. %s: cannot newlocale()\n", _ts);
	_t1(8, _td1_utf8, 0, 0);
}
#endif /* TEXT_SUPPORT_XLOCALE */

static char const *
_tcasego(char const *s1, size_t s1len, char const *s2, size_t s2len)
{ (void)s1len; (void)s2len; return txtcasestr(s1, s2); }

ATF_TC_BODY(txtcasestr_go, tc)
{
	_tptf = &_tcasego;

	_ts = "POSIX:txtcasestr";
	if (setlocale(LC_CTYPE, "POSIX") == 0)
		atf_tc_fail("1. %s: cannot set locale\n", _ts);
	_t1(9, _td1_posix, 1, 0);
	_t1(10, _td2_posix, 1, 0);

	_ts = "en_USSR.UTF-8:txtcasestr";
	if (setlocale(LC_CTYPE, "en_US.UTF-8") == 0)
		atf_tc_fail("1. %s: cannot set locale\n", _ts);
	_t1(11, _td1_utf8, 1, 0);
	_t1(12, _td2_utf8, 1, 0);
}

#ifdef TEXT_SUPPORT_XLOCALE
static char const *
_tcasego_l(char const *s1, size_t s1len, char const *s2, size_t s2len)
{ (void)s1len; (void)s2len; return txtcasestr_l(s1, s2, _lc); }

ATF_TC_BODY(txtcasestr_l_go, tc)
{
	_tptf = &_tcasego_l;

	_ts = "POSIX:txtcasestr_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "POSIX", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("2. %s: cannot newlocale()\n", _ts);
	_t1(13, _td1_posix, 1, 0);
	_t1(14, _td2_posix, 1, 0);

	_ts = "en_USSR.UTF-8:txtcasestr_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "en_US.UTF-8", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("2. %s: cannot newlocale()\n", _ts);
	_t1(15, _td1_utf8, 1, 0);
	_t1(16, _td2_utf8, 1, 0);
}
#endif /* TEXT_SUPPORT_XLOCALE */

static char const *
_tllcasego(char const *s1, size_t s1len, char const *s2, size_t s2len)
{ return txtllcasestr(s1, s1len, s2, s2len); }

ATF_TC_BODY(txtllcasestr_go, tc)
{
	_tptf = &_tllcasego;

	_ts = "POSIX:txtllcasestr";
	if (setlocale(LC_CTYPE, "POSIX") == 0)
		atf_tc_fail("3. %s: cannot set locale\n", _ts);
	_t1(17, _td2_posix, 0, 0);

	_ts = "en_USSR.UTF-8:txtllcasestr";
	if (setlocale(LC_CTYPE, "en_US.UTF-8") == 0)
		atf_tc_fail("3. %s: cannot set locale\n", _ts);
	_t1(18, _td2_utf8, 0, 0);
}

#ifdef TEXT_SUPPORT_XLOCALE
static char const *
_tllcasego_l(char const *s1, size_t s1len, char const *s2, size_t s2len)
{ return txtllcasestr_l(s1, s1len, s2, s2len, _lc); }

ATF_TC_BODY(txtllcasestr_l_go, tc)
{
	_tptf = &_tllcasego_l;

	_ts = "POSIX:txtllcasestr_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "POSIX", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("4. %s: cannot newlocale()\n", _ts);
	_t1(19, _td2_posix, 0, 0);

	_ts = "en_USSR.UTF-8:txtllcasestr_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "en_US.UTF-8", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("4. %s: cannot newlocale()\n", _ts);
	_t1(20, _td2_utf8, 0, 0);
}
#endif /* TEXT_SUPPORT_XLOCALE */

static char const *
_trgo(char const *s1, size_t s1len, char const *s2, size_t s2len)
{ (void)s1len; (void)s2len; return txtrstr(s1, s2); }

ATF_TC_BODY(txtrstr_go, tc)
{
	_tptf = &_trgo;

	_ts = "POSIX:txtrstr";
	if (setlocale(LC_CTYPE, "POSIX") == 0)
		atf_tc_fail("1. %s: cannot set locale\n", _ts);
	_t1(21, _td1_posix, 1, 1);

	_ts = "en_USSR.UTF-8:txtrstr";
	if (setlocale(LC_CTYPE, "en_US.UTF-8") == 0)
		atf_tc_fail("1. %s: cannot set locale\n", _ts);
	_t1(22, _td1_utf8, 1, 1);
}

#ifdef TEXT_SUPPORT_XLOCALE
static char const *
_trgo_l(char const *s1, size_t s1len, char const *s2, size_t s2len)
{ (void)s1len; (void)s2len; return txtrstr_l(s1, s2, _lc); }

ATF_TC_BODY(txtrstr_l_go, tc)
{
	_tptf = &_trgo_l;

	_ts = "POSIX:txtrstr_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "POSIX", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("2. %s: cannot newlocale()\n", _ts);
	_t1(23, _td1_posix, 1, 1);

	_ts = "en_USSR.UTF-8:txtrstr_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "en_US.UTF-8", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("2. %s: cannot newlocale()\n", _ts);
	_t1(24, _td1_utf8, 1, 1);
}
#endif /* TEXT_SUPPORT_XLOCALE */

static char const *
_tllrgo(char const *s1, size_t s1len, char const *s2, size_t s2len)
{ return txtllrstr(s1, s1len, s2, s2len); }

ATF_TC_BODY(txtllrstr_go, tc)
{
	_tptf = &_tllrgo;

	_ts = "POSIX:txtllrstr";
	if (setlocale(LC_CTYPE, "POSIX") == 0)
		atf_tc_fail("3. %s: cannot set locale\n", _ts);
	_t1(25, _td1_posix, 0, 1);

	_ts = "en_USSR.UTF-8:txtllrstr";
	if (setlocale(LC_CTYPE, "en_US.UTF-8") == 0)
		atf_tc_fail("3. %s: cannot set locale\n", _ts);
	_t1(26, _td1_utf8, 0, 1);
}

#ifdef TEXT_SUPPORT_XLOCALE
static char const *
_tllrgo_l(char const *s1, size_t s1len, char const *s2, size_t s2len)
{ return txtllrstr_l(s1, s1len, s2, s2len, _lc); }

ATF_TC_BODY(txtllrstr_l_go, tc)
{
	_tptf = &_tllrgo_l;

	_ts = "POSIX:txtllrstr_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "POSIX", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("4. %s: cannot newlocale()\n", _ts);
	_t1(27, _td1_posix, 0, 1);

	_ts = "en_USSR.UTF-8:txtllrstr_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "en_US.UTF-8", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("4. %s: cannot newlocale()\n", _ts);
	_t1(28, _td1_utf8, 0, 1);
}
#endif /* TEXT_SUPPORT_XLOCALE */

static char const *
_trcasego(char const *s1, size_t s1len, char const *s2, size_t s2len)
{ (void)s1len; (void)s2len; return txtrcasestr(s1, s2); }

ATF_TC_BODY(txtrcasestr_go, tc)
{
	_tptf = &_trcasego;

	_ts = "POSIX:txtrcasestr";
	if (setlocale(LC_CTYPE, "POSIX") == 0)
		atf_tc_fail("1. %s: cannot set locale\n", _ts);
	_t1(29, _td1_posix, 1, 1);
	_t1(30, _td2_posix, 1, 1);

	_ts = "en_USSR.UTF-8:txtrcasestr";
	if (setlocale(LC_CTYPE, "en_US.UTF-8") == 0)
		atf_tc_fail("1. %s: cannot set locale\n", _ts);
	_t1(31, _td1_utf8, 1, 1);
	_t1(32, _td2_utf8, 1, 1);
}

#ifdef TEXT_SUPPORT_XLOCALE
static char const *
_trcasego_l(char const *s1, size_t s1len, char const *s2, size_t s2len)
{ (void)s1len; (void)s2len; return txtrcasestr_l(s1, s2, _lc); }

ATF_TC_BODY(txtrcasestr_l_go, tc)
{
	_tptf = &_trcasego_l;

	_ts = "POSIX:txtrcasestr_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "POSIX", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("2. %s: cannot newlocale()\n", _ts);
	_t1(33, _td1_posix, 1, 1);
	_t1(34, _td2_posix, 1, 1);

	_ts = "en_USSR.UTF-8:txtrcasestr_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "en_US.UTF-8", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("2. %s: cannot newlocale()\n", _ts);
	_t1(35, _td1_utf8, 1, 1);
	_t1(36, _td2_utf8, 1, 1);
}
#endif /* TEXT_SUPPORT_XLOCALE */

static char const *
_tllrcasego(char const *s1, size_t s1len, char const *s2, size_t s2len)
{ return txtllrcasestr(s1, s1len, s2, s2len); }

ATF_TC_BODY(txtllrcasestr_go, tc)
{
	_tptf = &_tllrcasego;

	_ts = "POSIX:txtllrcasestr";
	if (setlocale(LC_CTYPE, "POSIX") == 0)
		atf_tc_fail("3. %s: cannot set locale\n", _ts);
	_t1(37, _td2_posix, 0, 1);

	_ts = "en_USSR.UTF-8:txtllrcasestr";
	if (setlocale(LC_CTYPE, "en_US.UTF-8") == 0)
		atf_tc_fail("3. %s: cannot set locale\n", _ts);
	_t1(38, _td2_utf8, 0, 1);
}

#ifdef TEXT_SUPPORT_XLOCALE
static char const *
_tllrcasego_l(char const *s1, size_t s1len, char const *s2, size_t s2len)
{ return txtllrcasestr_l(s1, s1len, s2, s2len, _lc); }

ATF_TC_BODY(txtllrcasestr_l_go, tc)
{
	_tptf = &_tllrcasego_l;

	_ts = "POSIX:txtllrcasestr_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "POSIX", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("4. %s: cannot newlocale()\n", _ts);
	_t1(39, _td2_posix, 0, 1);

	_ts = "en_USSR.UTF-8:txtllrcasestr_l";
	if ((_lc = newlocale(LC_CTYPE_MASK, "en_US.UTF-8", (locale_t)0)) ==
			(locale_t)0)
		atf_tc_fail("4. %s: cannot newlocale()\n", _ts);
	_t1(40, _td2_utf8, 0, 1);
}
#endif /* TEXT_SUPPORT_XLOCALE */
